@@ -3,11 +3,12 @@ package XML::Sig;
 use strict;
 use warnings;
 
+
 # use 'our' on v5.6.0
 use vars qw($VERSION @EXPORT_OK %EXPORT_TAGS $DEBUG);
 
 $DEBUG = 0;
-$VERSION = '0.34';
+$VERSION = '0.35';
 
 use base qw(Class::Accessor);
 XML::Sig->mk_accessors(qw(key));
@@ -24,6 +25,7 @@ use XML::LibXML;
 use MIME::Base64;
 use Carp;
 
+
 use constant TRANSFORM_ENV_SIG           => 'http://www.w3.org/2000/09/xmldsig#enveloped-signature';
 use constant TRANSFORM_C14N              => 'http://www.w3.org/TR/2001/REC-xml-c14n-20010315#';
 use constant TRANSFORM_C14N_COMMENTS     => 'http://www.w3.org/TR/2001/REC-xml-c14n-20010315#WithComments';
@@ -38,6 +40,9 @@ $SIG{INT} = sub { die "Interrupted\n"; };
 
 $| = 1;  # autoflush
 
+
+
+
 sub new {
     my $class = shift;
     my $params = shift;
@@ -64,6 +69,7 @@ sub new {
     return $self;
 }
 
+
 sub sign {
     my $self = shift;
     my ($xml) = @_;
@@ -153,7 +159,11 @@ sub sign {
             # concatonation of r and s in that order ($r . $s)
             my $r = $bin_signature->get_r;
             my $s = $bin_signature->get_s;
-            $signature        = encode_base64( $r . $s, "\n" );
+
+            my $rs = _zero_fill_buffer(320);
+            _concat_dsa_sig_r_s(\$rs, $r, $s);
+
+            $signature        = encode_base64( $rs, "\n" );
         } else {
             print ("    Signing SignedInfo using RSA key type\n") if $DEBUG;
             my $bin_signature = $self->{key_obj}->sign( $signed_info_canon );
@@ -172,6 +182,7 @@ sub sign {
     return $dom;
 }
 
+
 sub verify {
     my $self = shift;
     delete $self->{signer_cert};
@@ -332,11 +343,24 @@ sub verify {
     return 1;
 }
 
+
 sub signer_cert {
     my $self = shift;
     return $self->{signer_cert};
 }
 
+##
+## _get_ids_to_sign()
+##
+## Arguments:
+##
+## Returns: array Value of ID attributes from XML
+##
+## Finds all the values of the ID attributes in the XML
+## and return them in reverse order found.  Reverse order
+## assumes that the Signatures should be performed on lower
+## Nodes first.
+##
 sub _get_ids_to_sign {
     my $self = shift;
     my @id = $self->{parser}->findnodes('//@ID');
@@ -354,6 +378,17 @@ sub _get_ids_to_sign {
 
 }
 
+##
+## _get_xml_to_sign()
+##
+## Arguments:
+##    $id:     string ID of the Node for the XML to retrieve
+##
+## Returns: XML NodeSet to sign
+##
+## Find the XML node with the ID = $id and return the
+## XML NodeSet
+##
 sub _get_xml_to_sign {
     my $self = shift;
     my $id = shift;
@@ -364,6 +399,17 @@ sub _get_xml_to_sign {
     return $node;
 }
 
+##
+## _get_signed_xml($context)
+##
+## Arguments:
+##    $context:     string XML NodeSet used as context
+##
+## Returns: XML NodeSet for with ID equal to the URI
+##
+## Find the XML node with the ID = $URI and return the
+## XML NodeSet
+##
 sub _get_signed_xml {
     my $self = shift;
     my ($context) = @_;
@@ -377,6 +423,18 @@ sub _get_signed_xml {
     return $self->_get_node( $xpath, $context );
 }
 
+##
+## _transform($xml, $context)
+##
+## Arguments:
+##    $xml:     string XML NodeSet
+##    $context: string XML Context
+##
+## Returns: string  Transformed XML
+##
+## Canonicalizes/Transforms xml based on the Transforms
+## from the SignedInfo.
+##
 sub _transform {
     my $self = shift;
     my ($xml, $context) = @_;
@@ -415,6 +473,18 @@ sub _transform {
     return $xml;
 }
 
+##
+## _verify_rsa($context,$canonical,$sig)
+##
+## Arguments:
+##    $context:     string XML Context to use
+##    $canonical:   string Canonical XML to verify
+##    $sig:         string Base64 encode of RSA Signature
+##
+## Returns: integer (1 True, 0 False) if signature is valid
+##
+## Verify the RSA signature of Canonical XML
+##
 sub _verify_rsa {
     my $self = shift;
     my ($context,$canonical,$sig) = @_;
@@ -434,6 +504,17 @@ sub _verify_rsa {
     return 0;
 }
 
+##
+## _clean_x509($cert)
+##
+## Arguments:
+##    $cert:     string Certificate in base64 from XML
+##
+## Returns: string  Certificate in Valid PEM format
+##
+## Reformats Certifcate string into PEM format 64 characters
+## with proper header and footer
+##
 sub _clean_x509 {
     my $self = shift;
     my ($cert) = @_;
@@ -457,6 +538,18 @@ sub _clean_x509 {
     return $cert;
 }
 
+##
+## _verify_x509($context,$canonical,$sig)
+##
+## Arguments:
+##    $context:     string XML Context to use
+##    $canonical:   string Canonical XML to verify
+##    $sig:         string Base64 encode of RSA Signature
+##
+## Returns: integer (1 True, 0 False) if signature is valid
+##
+## Verify the RSA signature of Canonical XML using an X509
+##
 sub _verify_x509 {
     my $self = shift;
     my ($context,$canonical,$sig) = @_;
@@ -477,6 +570,18 @@ sub _verify_x509 {
     return $self->_verify_x509_cert($cert, $canonical, $sig);
 }
 
+##
+## _verify_x509_cert($cert,$canonical,$sig)
+##
+## Arguments:
+##    $cert:        string X509 Certificate
+##    $canonical:   string Canonical XML to verify
+##    $sig:         string Base64 encode of RSA Signature
+##
+## Returns: integer (1 True, 0 False) if signature is valid
+##
+## Verify the X509 signature of Canonical XML
+##
 sub _verify_x509_cert {
     my $self = shift;
     my ($cert, $canonical, $sig) = @_;
@@ -500,6 +605,66 @@ sub _verify_x509_cert {
     return 0;
 }
 
+##
+## _zero_fill_buffer($bits)
+##
+## Arguments:
+##    $bits:     number of bits to set to zero
+##
+## Returns: Zero filled bit buffer of size $bits
+##
+## Create a buffer with all bits set to 0
+##
+sub _zero_fill_buffer {
+    my $bits = shift;
+    # set all bit to zero
+    my $v = '';
+    for (my $i = 0; $i < $bits; $i++) {
+        vec($v, $i, 1) = 0;
+    }
+    return $v;
+}
+
+##
+## _concat_dsa_sig_r_s(\$buffer,$r,$s)
+##
+## Arguments:
+##    $buffer:      Zero Filled bit buffer
+##    $r:           octet stream
+##    $s:           octet stream
+##
+## Combine r and s components of DSA signature
+##
+sub _concat_dsa_sig_r_s {
+
+    my ($buffer, $r, $s) = @_;
+    my $bits_r = (length($r)*8)-1;
+    my $bits_s = (length($s)*8)-1;
+
+    # Place $s right justified in $v starting at bit 319
+    for (my $i = $bits_s; $i >=0; $i--) {
+        vec($$buffer, 160 + $i + (159 - $bits_s) , 1) = vec($s, $i, 1);
+    }
+
+    # Place $r right justified in $v starting at bit 159
+    for (my $i = $bits_r; $i >= 0 ; $i--) {
+        vec($$buffer, $i + (159 - $bits_r) , 1) = vec($r, $i, 1);
+    }
+
+}
+
+##
+## _verify_dsa($context,$canonical,$sig)
+##
+## Arguments:
+##    $context:     string XML Context to use
+##    $canonical:   string Canonical XML to verify
+##    $sig:         string Base64 encode 40 byte string of r and s
+##
+## Returns: integer (1 True, 0 False) if signature is valid
+##
+## Verify the DSA signature of Canonical XML
+##
 sub _verify_dsa {
     my $self = shift;
     my ($context,$canonical,$sig) = @_;
@@ -527,7 +692,7 @@ sub _verify_dsa {
     # The signature value consists of the base64 encoding of the
     # concatonation of r and s in that order ($r . $s)
     # Binary Signature is stored as a concatonation of r and s
-    my ($r, $s) = unpack('a20 a20', $bin_signature);
+    my ($r, $s) = unpack('a20a20', $bin_signature);
 
     # Create a new Signature Object from r and s
     my $sigobj = Crypt::OpenSSL::DSA::Signature->new();
@@ -539,6 +704,17 @@ sub _verify_dsa {
     return 0;
 }
 
+##
+## _get_node($xpath, context)
+##
+## Arguments:
+##    $xpath:       string XML XPath to use
+##    $context:     string XML context
+##
+## Returns: string  XML NodeSet
+##
+## Return a NodeSet based on the xpath string
+##
 sub _get_node {
     my $self = shift;
     my ($xpath, $context) = @_;
@@ -582,6 +758,16 @@ sub _transform_env_sig {
     return $str;
 }
 
+##
+## _trim($string)
+##
+## Arguments:
+##    $string:      string String to remove whitespace
+##
+## Returns: string  Trimmed String
+##
+## Trim the whitespace from the begining and end of the string
+##
 sub _trim {
     my $string = shift;
     $string =~ s/^\s+//;
@@ -589,6 +775,19 @@ sub _trim {
     return $string;
 }
 
+##
+## _load_dsa_key($key_text)
+##
+## Arguments:
+##    $key_text:    string DSA Private Key as String
+##
+## Returns: nothing
+##
+## Populate:
+##   self->{KeyInfo}
+##   self->{key_obj}
+##   self->{key_type}
+##
 sub _load_dsa_key {
     my $self = shift;
     my $key_text = shift;
@@ -625,7 +824,19 @@ sub _load_dsa_key {
     }
 }
 
-
+##
+## _load_rsa_key($key_text)
+##
+## Arguments:
+##    $key_text:    string RSA Private Key as String
+##
+## Returns: nothing
+##
+## Populate:
+##   self->{KeyInfo}
+##   self->{key_obj}
+##   self->{key_type}
+##
 sub _load_rsa_key {
     my $self = shift;
     my ($key_text) = @_;
@@ -664,6 +875,18 @@ sub _load_rsa_key {
     }
 }
 
+##
+## _load_x509_key($key_text)
+##
+## Arguments:
+##    $key_text:    string RSA Private Key as String
+##
+## Returns: nothing
+##
+## Populate:
+##   self->{key_obj}
+##   self->{key_type}
+##
 sub _load_x509_key {
     my $self = shift;
     my $key_text = shift;
@@ -684,6 +907,18 @@ sub _load_x509_key {
     }
 }
 
+##
+## _load_cert_file()
+##
+## Arguments: none
+##
+## Returns: nothing
+##
+## Read the file name from $self->{ cert } and
+## Populate:
+##   self->{key_obj}
+##   $self->{KeyInfo}
+##
 sub _load_cert_file {
     my $self = shift;
 
@@ -718,6 +953,18 @@ sub _load_cert_file {
     return;
 }
 
+##
+## _load_cert_text()
+##
+## Arguments: none
+##
+## Returns: nothing
+##
+## Read the certificate from $self->{ cert_text } and
+## Populate:
+##   self->{key_obj}
+##   $self->{KeyInfo}
+##
 sub _load_cert_text {
     my $self = shift;
 
@@ -742,6 +989,15 @@ sub _load_cert_text {
     return;
 }
 
+##
+## _load_key($file)
+##
+## Arguments: $self->{ key }
+##
+## Returns: nothing
+##
+## Load the key and process it acording to its headers
+##
 sub _load_key {
     my $self = shift;
     my $file = $self->{ key };
@@ -779,6 +1035,17 @@ sub _load_key {
     return;
 }
 
+##
+## _signature_xml($signed_info,$signature_value)
+##
+## Arguments:
+##   $signed_info:      string XML String Fragment
+##   $signature_value   String Base64 Signature Value
+##
+## Returns: string      XML fragment
+##
+## Create a XML string of the Signature
+##
 sub _signature_xml {
     my $self = shift;
     my ($signed_info,$signature_value) = @_;
@@ -789,6 +1056,16 @@ sub _signature_xml {
         </dsig:Signature>};
 }
 
+##
+## _signedinfo_xml($digest_xml)
+##
+## Arguments:
+##   $digest_xml        string XML String Fragment
+##
+## Returns: string      XML fragment
+##
+## Create a XML string of the SignedInfo
+##
 sub _signedinfo_xml {
     my $self = shift;
     my ($digest_xml) = @_;
@@ -801,6 +1078,17 @@ sub _signedinfo_xml {
             </dsig:SignedInfo>};
 }
 
+##
+## _reference_xml($id)
+##
+## Arguments:
+##   $id        string XML ID related to the URI
+##   $digest    string Base64 encoded digest
+##
+## Returns: string      XML fragment
+##
+## Create a XML string of the Reference
+##
 sub _reference_xml {
     my $self = shift;
     my $id = shift;
@@ -815,6 +1103,19 @@ sub _reference_xml {
                     </dsig:Reference>};
 }
 
+
+##
+## _canonicalize_xml($xml, $context)
+##
+## Arguments:
+##    $xml:     string XML NodeSet
+##    $context: string XML Context
+##
+## Returns: string  Canonical XML
+##
+## Canonicalizes xml based on the CanonicalizationMethod
+## from the SignedInfo.
+##
 sub _canonicalize_xml {
     my $self = shift;
     my ($xml, $context) = @_;
@@ -870,7 +1171,7 @@ XML::Sig
 
 =head1 VERSION
 
-version 0.34
+version 0.35
 
 =head1 SYNOPSIS
 
@@ -891,17 +1192,147 @@ version 0.34
 =head1 DESCRIPTION
 
 This perl module provides two primary capabilities: given an XML string, create
-and insert a digital signature, or if one is already present in the string verify
+and insert digital signatures, or if one is already present in the string verify
 it -- all in accordance with the W3C standard governing XML signatures.
 
-=item _canonicalize_xml($xml, $context)
-
-Canonicalizes xml based on the CanonicalizationMethod
-
 =head1 NAME
 
 XML::Sig - A toolkit to help sign and verify XML Digital Signatures.
 
+=head1 PREREQUISITES
+
+=over
+
+=item L<Digest::SHA>
+=item L<XML::LibXML>
+=item L<MIME::Base64>
+=item L<Crypt::OpenSSL::X509>
+=item L<Crypt::OpenSSL::Bignum>
+=item L<Crypt::OpenSSL::RSA>
+=item L<Crypt::OpenSSL::DSA>
+
+=back
+
+=head1 USAGE
+
+=head2 SUPPORTED ALGORITHMS & TRANSFORMS
+
+This module supports the following signature methods:
+
+=over
+
+=item DSA
+=item RSA
+=item RSA encoded as x509
+
+=back
+
+This module supports the following canonicalization methods and transforms:
+
+=over
+
+=item Enveloped Signature
+=item REC-xml-c14n-20010315#
+=item REC-xml-c14n-20010315#WithComments
+=item REC-xml-c14n11-20080502
+=item REC-xml-c14n11-20080502#WithComments
+=item xml-exc-c14n#
+=item xml-exc-c14n#WithComments
+
+=back
+
+=head2 OPTIONS
+
+Each of the following options are also accessors on the main
+XML::Sig object. TODO Not strictly correct rewrite
+
+=over
+
+=item B<key>
+
+The path to a file containing the contents of a private key. This option
+is used only when generating signatures.
+
+=item B<cert>
+
+The path to a file containing a PEM-formatted X509 certificate. This
+option is used only when generating signatures with the "x509"
+option. This certificate will be embedded in the signed document, and
+should match the private key used for the signature.
+
+=item B<cert_text>
+
+A string containing a PEM-formatted X509 certificate. This
+option is used only when generating signatures with the "x509"
+option. This certificate will be embedded in the signed document, and
+should match the private key used for the signature.
+
+=item B<x509>
+
+Takes a true (1) or false (0) value and indicates how you want the
+signature to be encoded. When true, the X509 certificate supplied will
+be encoded in the signature. Otherwise the native encoding format for
+RSA and DSA will be used.
+
+=back
+
+=head2 METHODS
+
+=head3 B<new(...)>
+
+Constructor; see OPTIONS above.
+
+=head3 B<sign($xml)>
+
+When given a string of XML, it will return the same string with a signature
+generated from the key provided when the XML::Sig object was initialized.
+
+This method will sign all elements in your XML with an ID (case sensitive)
+attribute. Each element with an ID attribute will be the basis for a seperate
+signature. It will correspond to the URI attribute in the Reference element
+that will be contained by the signature. If no ID attribute can be found on
+an element, the signature will not be created.
+
+The elements are signed in reverse order currently assuming (possibly
+incorrectly) that the lower element in the tree may need to be signed
+inclusive of its Signature because it is a child of the higher element.
+
+Arguments:
+    $xml:     string XML string
+
+Returns: string  Signed XML
+
+=head3 B<verify($xml)>
+
+Returns true or false based upon whether the signature is valid or not.
+
+When using XML::Sig exclusively to verify a signature, no key needs to be
+specified during initialization given that the public key should be
+transmitted with the signature.
+
+XML::Sig checks all signature in the provided xml and will fail should any
+signature pointing to an existing ID in the XML fail to verify.
+
+Should there be a Signature included that does not point to an existing node
+in the XML it is ignored and other Signaures are checked.  If there are no
+other Signatures it will return false.
+
+Arguments:
+    $xml:     string XML string
+
+Returns: string  Signed XML
+
+=head3 B<signer_cert()>
+
+Following a successful verify with an X509 certificate, returns the
+signer's certificate as embedded in the XML document for verification
+against a CA certificate. The certificate is returned as a
+Crypt::OpenSSL::X509 object.
+
+Arguments: none
+
+Returns: Crypt::OpenSSL::X509: Certificate used to sign the XML
+
 =head1 ABOUT DIGITAL SIGNATURES
 
 Just as one might want to send an email message that is cryptographically signed
@@ -990,131 +1421,6 @@ Now, let's insert a signature:
     </Signature>
   </foo>
 
-=head1 PREREQUISITES
-
-=over
-
-=item L<Digest::SHA>
-
-=item L<XML::LibXML>
-
-=item L<MIME::Base64>
-
-=item L<Crypt::OpenSSL::X509>
-
-=item L<Crypt::OpenSSL::Bignum>
-
-=item L<Crypt::OpenSSL::RSA>
-
-=item L<Crypt::OpenSSL::DSA>
-
-=item L<Crypt::OpenSSL::X509>
-
-=back
-
-=head1 USAGE
-
-=head2 SUPPORTED ALGORITHMS & TRANSFORMS
-
-This module supports the following signature methods:
-
-=over
-
-=item DSA
-
-=item RSA
-
-=item RSA encoded as x509
-
-=back
-
-This module supports the following canonicalization methods and transforms:
-
-=over
-
-=item REC-xml-c14n-20010315#
-
-=item REC-xml-c14n-20010315#WithComments
-
-=item REC-xml-c14n11-20080502
-
-=item REC-xml-c14n11-20080502#WithComments
-
-=item xml-exc-c14n#
-
-=item xml-exc-c14n#WithComments
-
-=item Enveloped Signature
-
-=back
-
-=head2 METHODS
-
-=over
-
-=item B<new(...)>
-
-Constructor; see OPTIONS below.
-
-=item B<sign($xml)>
-
-When given a string of XML, it will return the same string with a signature
-generated from the key provided when the XML::Sig object was initialized.
-
-This method presumes that there is one and only one element in your XML
-document with an ID (case sensitive) attribute. This is the element that will
-be the basis for the signature. It will also correspond to the URI attribute
-in the Reference element that will be contained by the signature. If no ID
-attribute can be found on an element, the signature will not be created.
-
-=item B<verify($xml)>
-
-Returns true or false based upon whether the signature is valid or not.
-
-When using XML::Sig exclusively to verify a signature, no key needs to be
-specified during initialization given that the public key should be
-transmitted with the signature.
-
-XML::Sig checks all signature in the provided xml and will fail should any
-signature pointing to an existing ID in the XML fail to verify
-
-=item B<signer_cert()>
-
-Following a successful verify with an X509 certificate, returns the
-signer's certificate as embedded in the XML document for verification
-against a CA certificate. The certificate is returned as a
-Crypt::OpenSSL::X509 object.
-
-=back
-
-=head2 OPTIONS
-
-Each of the following options are also accessors on the main
-File::Download object.
-
-=over
-
-=item B<key>
-
-The path to a file containing the contents of a private key. This option
-is used only when generating signatures.
-
-=item B<cert>
-
-The path to a file containing a PEM-formatted X509 certificate. This
-option is used only when generating signatures with the "x509"
-option. This certificate will be embedded in the signed document, and
-should match the private key used for the signature.
-
-=item B<x509>
-
-Takes a true (1) or false (0) value and indicates how you want the
-signature to be encoded. When true, the X509 certificate supplied will
-be encoded in the signature. Otherwise the native encoding format for
-RSA and DSA will be used.
-
-=back
-
 =head1 SEE ALSO
 
 L<http://www.w3.org/TR/xmldsig-core/>
@@ -1141,7 +1447,7 @@ Original Author: Byrne Reese <byrne@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Byrne Reese, Chris Andrews and Others; in detail:
+This software is copyright (c) 2021 by Byrne Reese, Chris Andrews and Others; in detail:
 
   Copyright 2009       Byrne, Michael Hendricks
             2010       Chris Andrews
@@ -1150,7 +1456,7 @@ This software is copyright (c) 2020 by Byrne Reese, Chris Andrews and Others; in
             2015       Mike Wisener
             2016       Jeff Fearn
             2017       Mike Wisener, xmikew
-            2019-2020  Timothy Legge
+            2019-2021  Timothy Legge
 
 
 This is free software; you can redistribute it and/or modify it under
